/***************************************************
 *                                                 *
 *    Macro Assembler			           *
 *                                                 *
 *    Written by Richard Cornwell                  *
 *               3/20/86                           *
 *                                                 *
 ***************************************************/

#include "macro.h"

struct opcodes optab[] = {
	"abx",		TZER,	0x3a,
	"adca",		TOPR,	0x89,
	"adcb",		TOPR,	0xC9,
	"adda",		TOPR,	0x8b,
	"addb",		TOPR,	0xcb,
	"addd",		TSIX,	0xc3,
	"anda",		TOPR,	0x84,
	"andb",		TOPR,	0xc4,
	"andcc",	TBYT,	0x1c,
	"ascii",	ASCII,	0,
	"asciz",	ASCIZ,	0,
	"asl",		TMEM,	0x08,
	"asla",		TZER,	0x48,
	"aslb",		TZER,	0x58,
	"asr",		TMEM,	0x07,
	"asra",		TZER,	0x47,
	"asrb",		TZER,	0x57,
	"bcc",		TBRS,	0x24,
	"bcs",		TBRS,	0x25,
	"beq",		TBRS,	0x27,
	"bge",		TBRS,	0x2c,
	"bgt",		TBRS,	0x2e,
	"bhi",		TBRS,	0x22,
	"bhs",		TBRS,	0x24,
	"bita",		TOPR,	0x85,
	"bitb",		TOPR,	0xc5,
	"ble",		TBRS,	0x2f,
	"blo",		TBRS,	0x25,
	"bls",		TBRS,	0x23,
	"blt",		TBRS,	0x2d,
	"bmi",		TBRS,	0x2b,
	"bne",		TBRS,	0x26,
	"bpl",		TBRS,	0x2a,
	"bra",		TBRS,	0x20,
	"brn",		TBRS,	0x21,
	"bsr",		TBRS,	0x8d,
	"bss",		BSS,	0,
	"bvc",		TBRS,	0x28,
	"bvs",		TBRS,	0x29,
	"byte",		BYTE,	0,
	"clc",		TZER,	0x1c7f,		/* andcc   #07fh */
	"clf",		TZER,	0x1cbf,		/* andcc   #0bfh */
	"cli",		TZER,	0x1cef,		/* andcc   #0efh */
	"clif",		TZER,	0x1caf,		/* andcc   #0afh */
	"clr",		TMEM,	0x0f,
	"clra",		TZER,	0x4f,
	"clrb",		TZER,	0x5f,
	"clrv",		TZER,	0x1cfd,		/* andcc   #0fdh */
	"cmpa",		TOPR,	0x81,
	"cmpb",		TOPR,	0xc1,
	"cmpd",		TOPR,	0x1083,
	"cmps",		TOPR,	0x118c,
	"cmpu",		TOPR,	0x1183,
	"cmpx",		TOPR,	0x8c,
	"cmpy",		TOPR,	0x108c,
	"com",		TMEM,	0x03,
	"coma",		TZER,	0x43,
	"comb",		TZER,	0x53,
	"comm",		COMM,	0,
	"cwai",		TBYT,	0x3c,
	"daa",		TZER,	0x19,
	"data",		DSECT,	0,
	"dec",		TMEM,	0x0a,
	"deca",		TZER,	0x4a,
	"decb",		TZER,	0x5a,
	"des",		TZER,	0x327f,		/* leas   -1,s */
	"dex",		TZER,	0x301f,		/* leax   -1,x */
	"dey",		TZER,	0x313f,		/* leay   -1,y */
	"end",		END,	0,
#ifdef MAXCOND
	"endc",		ENDC,	0,
#endif
	"entry",	ENTRY,	0,
	"eora",		TOPR,	0x88,
	"eorb",		TOPR,	0xc8,
	"equ",		EQU,	0,
	"even",		EVEN,	0,
	"exg",		TREG,	0x1e,
	"extern",	EXTERN,	0,
	"globl",	GLOBAL,	0,
#ifdef MAXCOND
	"if",		IF,	0,
	"ifd",		IFD,	0,
	"ife",		IFE,	0,
	"iff",		IFF,	0,
	"ifn",		IFN,	0,
	"ift",		IFT,	0,
	"iftf",		IFTF,	0,
	"ifu",		IFU,	0,
#endif
	"inc",		TMEM,	0x0c,
	"inca",		TZER,	0x4c,
	"incb",		TZER,	0x5c,
	"ins",		TZER,	0x3261,		/* leas   1,s */
	"inx",		TZER,	0x3001,		/* leax   1,x */
	"iny",		TZER,	0x3121,		/* leay   1,y */
	"jmp",		TMEM,	0x0e,
	"jsr",		TMEM,	0x9d,
	"lbcc",		TBRL,	0x1024,
	"lbcs",		TBRL,	0x1025,
	"lbeq",		TBRL,	0x1027,
	"lbge",		TBRL,	0x102c,
	"lbgt",		TBRL,	0x102e,
	"lbhi",		TBRL,	0x1022,
	"lbhs",		TBRL,	0x1024,
	"lble",		TBRL,	0x102f,
	"lblo",		TBRL,	0x1025,
	"lbls",		TBRL,	0x1023,
	"lblt",		TBRL,	0x102d,
	"lbmi",		TBRL,	0x102b,
	"lbne",		TBRL,	0x1026,
	"lbpl",		TBRL,	0x102a,
	"lbra",		TBRL,	0x16,
	"lbrn",		TBRL,	0x1021,
	"lbsr",		TBRL,	0x17,
	"lbvc",		TBRL,	0x1028,
	"lbvs",		TBRL,	0x1029,
	"lda",		TOPR,	0x86,
	"ldb",		TOPR,	0xc6,
	"ldd",		TSIX,	0xcc,
	"lds",		TSIX,	0x10ce,
	"ldu",		TSIX,	0xce,
	"ldx",		TSIX,	0x8e,
	"ldy",		TSIX,	0x108e,
	"leas",		TLEA,	0x32,
	"leau",		TLEA,	0x33,
	"leax",		TLEA,	0x30,
	"leay",		TLEA,	0x31,
	"lsl",		TMEM,	0x08,
	"lsla",		TZER,	0x48,
	"lslb",		TZER,	0x58,
	"lsr",		TMEM,	0x04,
	"lsra",		TZER,	0x44,
	"lsrb",		TZER,	0x54,
#ifdef MACASM
	"macro",	MACRO,	0,
	"mend",		MEND,	0,
	"mglb",		MGLB,	0,
	"mgo",		MGO,	0,
	"mlcl",		MLCL,	0,
	"msetc",	MSET,	0,
	"msetl",	MSETL,	0,
#endif 
	"mul",		TZER,	0x3d,
	"neg",		TMEM,	0x00,
	"nega",		TZER,	0x40,
	"negb",		TZER,	0x50,
	"nop",		TZER,	0x12,
	"ora",		TOPR,	0x8a,
	"orb",		TOPR,	0xca,
	"orcc",		TBYT,	0x1a,
	"page",		PAGE,	0,
	"print",	PRINT,	0,
	"pshs",		TSTK,	0x34,
	"pshu",		TSTK,	0x36,
	"puls",		TSTK,	0x35,
	"pulu",		TSTK,	0x37,
	"rol",		TMEM,	0x09,
	"rola",		TZER,	0x49,
	"rolb",		TZER,	0x59,
	"ror",		TMEM,	0x06,
	"rora",		TZER,	0x46,
	"rorb",		TZER,	0x56,
	"rmb",		BLOCK,	0,
	"rti",		TZER,	0x3b,
	"rts",		TZER,	0x39,
	"sbca",		TOPR,	0x82,
	"sbcb",		TOPR,	0xc2,
	"sec",		TZER,	0x1a01,		/* orcc   #001h */
	"sef",		TZER,	0x1a40,		/* orcc   #040h */
	"sei",		TZER,	0x1a10,		/* orcc   #010h */
	"seif",		TZER,	0x1a50,		/* orcc   #050h */
	"sev",		TZER,	0x1a02,		/* orcc   #002h */
	"sex",		TZER,	0x1d,
	"space",	SPACE,	0,
	"sta",		TSTA,	0x97,
	"stb",		TSTA,	0xd7,
	"std",		TSTA,	0xdd,
	"stitle",	STITLE,	0,
	"sts",		TSTA,	0x10df,
	"stu",		TSTA,	0xdf,
	"stx",		TSTA,	0x9f,
	"suba",		TOPR,	0x80,
	"subb",		TOPR,	0xc0,
	"subd",		TSIX,	0x83,
	"swi",		TZER,	0x3f,
	"swi2",		TZER,	0x103f,
	"swi3",		TZER,	0x113f,
	"sync",		TZER,	0x13,
	"text",		CSECT,	0,
	"tfr",		TREG,	0x1f,
	"title",	TITLE,	0,
	"tst",		TMEM,	0x0d,
	"tsta",		TZER,	0x4d,
	"tstb",		TZER,	0x5d,
	"wai",		TZER,	0x3cff,			/* cwai    #0ffh */
	"word",		WORD,	0,
	};

int opsize = sizeof(optab)/sizeof(struct opcodes);

struct opcodes reglist[] = {
	"d",		0x003,	REG,
	"x",		0x110,	REG,
	"y",		0x220,	REG,
	"u",		0x340,	REG,
	"s",		0x440,	REG,
	"pc",		0x580,	REG,
	"a",		0x802,	REG,
	"b",		0x904,	REG,
	"cc",		0xa01,	REG,
	"dp",		0xb08,	REG,
	NULL,		0,	0 };
