/***************************************************
 *                                                 *
 *    Macro Assembler				   *
 *                                                 *
 *    Copyright 1989    Richard P. Cornwell        *
 *    Developmental Code not for redistubution     *
 *                                                 *
 *    Written by Richard Cornwell                  *
 *               3/20/86                           *
 *                                                 *
 ***************************************************/


#include <time.h>
#include "a.out.h"
#include <stdio.h>
#ifdef I386
#include "i386.h"
#else
#include "machine.h"
#endif

#ifdef __STDC__
#define _PROTOTYPE(function, params)    function params
#else
#define _PROTOTYPE(function, params)    function()
#endif

#define MAXCOND 10			/* Turn on condition assembly */
#define MACASM				/* Turn on macro assembly */
#define MAXNEST 10
#define NOFLOAT				/* Turn off FLOAT numbers */

/* This is the layout of instructions in the opcode table. */
struct opcodes {
	char *opc;			/* name of opcode */
	short int type;			/* Opcode class */
	unsigned short int base;	/* Base for this one */
	} ;

/* Expression output */
struct ex {
	unsigned long val;
	struct symbol *sym;
	unsigned int mode;
	};

#define NCPS  8
#define TXTSIZE 60
#define MACNEST 10

/* Symbol table format */
struct symbol {
	char *name;			/* Name */
	long int sindex;		/* Index into the string table */
	long unsigned value;		/* Value of symbol */
	long int index;			/* Index of symbol in symbol table */
	unsigned short int flag;	/* Flags */
	short desc;			/* Used by stab */
	struct symbol *next;		/* Pointer to next defined symbol */
	struct symbol *rlink,*llink;	/* Pointer for lookup */
	char other;			/* Used by stab */
	}  ;

/* Define a linked list of lines we want ot assemble as short instruction
   if possible */
struct shortop {
	long int loc;			/* Location of the instruction */
	struct symbol *sym;		/* Pointer to referenced symbol */
	char flags;			/* Flags */
	char size;			/* Current size */
	short int extra;		/* Extra information for sizing */
	struct shortop *next;		/* Next one in chain */
};

#define XALIGN	0x80			/* Opcode is an alignment opcode */

#ifdef MACASM 
struct text { 			/* Macro symbol table */
	char *name;			/* Name of symbol */
	char data[TXTSIZE];		/* Value */
	struct text *ptr;		/* Pointer to next one */
	} ;

struct mac { 
	char *name;			/* Name of macro */
	char *def;			/* Pointer to defintion */
	char *sym;			/* Pointer to inital vars */
	struct mac *rlink, *llink;
	} ;

#endif

#define START 40
#define LPPG 55
#define DEF 1				/* Define symbol in stab */
#define NODEF 0				/* Don't put in stab */
#define COMMENT	'!'
#define LINSEP	';'

/* Pseudo Ops. */
#define CSECT	10
#define DSECT	11
#define WORD	12
#define BLOCK	13
#define BYTE	14
#define LONG	15
#define END	16
#define ENTRY	17
#define GLOBAL	18
#define EXTERN	19
#define ASCII	20
#define ASCIZ	21
#define EQU	22
#define TITLE	23
#define SPACE	24
#define PAGE	25
#define STITLE	26
#define PRINT	27
#define EVEN	28
#define BSS	29
#define COMM	30
#ifdef MACASM
#define MACRO	31
#define MEND	32
#define MSET	33
#define MSETL	34
#define MGO	35
#define MLCL	36
#define MGLB	37
#endif
#ifdef MAXCOND
#define ENDC	40
#define	IF	41
#define	IFD	42
#define	IFE	43
#define	IFF	44
#define	IFN	45
#define	IFT	46
#define	IFTF	47
#define	IFU	48
#endif
#define STABS	50
#define STABN	51
#define STABD	52
#define SET	53
#define LCOMM	54

/* Expression defines */
#define NUMBER	1
#define RPARN 	2
#define LPARN	3
#define COMMA	4
#define PLUS 	5
#define MINUS	6
#define TIMES	7
#define DIVIDE	8
#define SHR	9
#define SHL	10
#define AND	11
#define OR 	12
#define XOR	13
#define NOT	14
#define EOL	15
#define MOD	16
#define LE	17
#define LT	18
#define EQ	19
#define NE	20
#define GT	21
#define GE	22
#define NUM	23
#define LBRACE	24
#define RBRACE	25
#define AT	26
#define COLON	27
#define DOLLAR	28

extern unsigned long stkval[20];
extern struct symbol *stksym[20];
extern int oldlex[20];

extern int sp, oldptr;

#define	max(x,y)	(((x)>(y))?(x):(y))
#define	min(x,y)	(((x)<(y))?(x):(y))

#define MAXSYS	(55)

extern FILE *txtrel, *datrel, *wrkfil, *input, *output, *listfil;

extern struct symbol *symtab;
extern struct shortop *opchain;
extern struct tm *timev;

extern char line[1024];			/* for stab's use */
extern int position;
extern char timestamp[];
extern char prline[133];
extern char title[];
extern char stitle[];

extern char *opcfld;
extern char *labfld;
extern char *oprand;

#ifdef MINIX_OBJ
#define N_TYPE		N_SECT
#define X_TYPE		N_SECT
#define N_EXT		C_EXT
#define C_PCR	(C_STAT+C_STAT)
#define REG	(N_BSS+2)
#define P_UNDF	(&sects[N_UNDF])
#define P_ABS	(&sects[N_ABS])
#define P_TEXT	(&sects[N_TEXT])
#define P_DATA	(&sects[N_DATA])
#define P_BSS	(&sects[N_BSS])
#else
#define X_TYPE	(N_TYPE/2)
#define N_PCR	(1)
#define REG	(N_BSS+4)
#define P_UNDF	(&sects[N_UNDF/2])
#define P_ABS	(&sects[N_ABS/2])
#define P_TEXT	(&sects[N_TEXT/2])
#define P_DATA	(&sects[N_DATA/2])
#define P_BSS	(&sects[N_BSS/2])
#endif

extern struct symbol sects[X_TYPE];

#define N_ETYP	(N_TYPE&(~N_EXT))


extern int ecount, pass, opsize, errcnt, linnum, pos, endflag;
extern long int symsize;
extern int pgline, prflag, prlflag, prcol, startime;
extern int rflag, lflag, jflag, uflag, stor;
extern int csindex;

extern struct opcodes optab[], reglist[];
extern int size;
extern unsigned long loc[N_TYPE], txtloc, datloc, location,  oloc;
extern int section;
extern unsigned long int txsize;
extern long unsigned entry;
#ifdef MAXCOND
extern int currlvl, condval[];
_PROTOTYPE( int defed,  (char *p));
#endif

/* expr.c */
_PROTOTYPE(int expr, (register struct ex *p));
_PROTOTYPE(int orop, (void));
_PROTOTYPE(int xorop, (void));
_PROTOTYPE(int andop, (void));
_PROTOTYPE(int cmpop, (void));
_PROTOTYPE(int shfop, (void));
_PROTOTYPE(int addop, (void));
_PROTOTYPE(int mulop, (void));
_PROTOTYPE(void eval, (int op));
_PROTOTYPE(int value, (void));
_PROTOTYPE(int unlex, (int x));
_PROTOTYPE(int lex, (void));
/* main.c */
_PROTOTYPE(void main, (int argc, char *argv[]));
_PROTOTYPE(void errexit, (void));
_PROTOTYPE(void endpass, (void));
_PROTOTYPE(void fillloc, (unsigned long int v, int s));
_PROTOTYPE(void set, (char *s, char *p));
_PROTOTYPE(void printit, (void));
_PROTOTYPE(void prtstr, (char *s, FILE *f));
_PROTOTYPE(void prtdec, (long int n, FILE *f));
_PROTOTYPE(void prolog, (void));
_PROTOTYPE(void epilog, (void));
_PROTOTYPE(int opennext, (void));
_PROTOTYPE(void readline, (void));
_PROTOTYPE(void error, (int c));
_PROTOTYPE(void fatal, (char *s, char *p));
_PROTOTYPE(void putl, (register unsigned long i, register FILE *iop));
_PROTOTYPE(int putwd, (unsigned int i, register FILE *iop));
_PROTOTYPE(void copyfile, (register FILE *in, register FILE *out, register long int size));
/* symbol.c */
_PROTOTYPE(void enter, (char *p, unsigned long l, unsigned int f));
_PROTOTYPE(struct symbol *lookup, (char *p, int def));
_PROTOTYPE(int makestab, (char *p, unsigned int type, unsigned int other, unsigned int desc, unsigned long val));
_PROTOTYPE(int printsym, (void));
_PROTOTYPE(void traverse, (void));
_PROTOTYPE(int dosym, (void));
_PROTOTYPE(void fixsym, (void));
_PROTOTYPE(int initsym, (void));
_PROTOTYPE(void adjust, (long int after, long int change, long int sec));
_PROTOTYPE(void addchain, (long int loc, struct symbol *sym, int flag, int size, int extra));
_PROTOTYPE(void fixops, (void));
_PROTOTYPE(int defed, (char *p));
/* syntax.c */
_PROTOTYPE(void scan, (void));
_PROTOTYPE(int parse, (void));
_PROTOTYPE(int psuedo, (struct opcodes *ptr));
_PROTOTYPE(int unquote, (char *s));
_PROTOTYPE(struct opcodes *oplook, (char *str));
_PROTOTYPE(char relcode, (int m));
_PROTOTYPE(void genword, (unsigned long x, struct symbol *p, int sz, int pcrel));

#ifdef MACASM
extern struct mac *mactab;
char *macptr[MAXNEST];
extern struct text *macsym[], *global;
extern int macpos[], maclvl, macnest, mactype[], sysindx;
_PROTOTYPE( int addline, (void));
_PROTOTYPE( int expand, (void));
_PROTOTYPE( char macget, (void));
_PROTOTYPE( char macpeek, (void));
_PROTOTYPE( char *maclook, (char *s));
_PROTOTYPE( int macset, (char *n, char *v));
_PROTOTYPE( int maclcl, (void));
_PROTOTYPE( int macglb, (void));
_PROTOTYPE( int macgo, (char *n));
_PROTOTYPE( int macbuild, (struct mac *m));
_PROTOTYPE( int macadd, (void));
_PROTOTYPE( struct mac *macfind, (char *n));
_PROTOTYPE( struct mac *macent, (char *n));
_PROTOTYPE( int macend, (void));
_PROTOTYPE( int getline, (void));
#endif
